% Computes minimum delta values in Table 3.4 in Pearson, Pestana & 
% Silvester, 2016.
% based on: XINFSUP
%   IFISS scriptfile: DJS; 27 May 2015.
% Copyright (c) 2005 D.J. Silvester, H.C. Elman, A. Ramage
%
% Edited JJP: 22 Jan, 2016. Pull out delta part of EigTables.m and put 
% in table

% Adds necessary directories to Matlab path
thisdir = pwd;
addpath(genpath(thisdir));
setifisspath;
cd(ifisspath)
setpath;
cd(thisdir)

problist = {'S3_G5_R3'};%,'S5_G5'};
numprob = length(problist);
probname = {'Regularized cavity'};%,'Obstacle'};
elemlist = {'Q1Q1','Q1P0','Q2Q1'};
numelem = length(elemlist);
delmatfull = zeros(numprob,numelem);
delmatfull1 = zeros(numprob,numelem);
delmatdiag = zeros(numprob,numelem);
delmatdiag1 = zeros(numprob,numelem);
includebound = 1; % Include boundary nodes
diag_type = 'diag'; % diag or lump

for k = 1:numprob
    pb = problist{k};
    display(pb)
    for j = 1:numelem
        pbel = [pb,'_',elemlist{j}];
        batchmode_scalestokes(pbel); % Need this for obstacle prob
        close all;
        load batchrun
        cd(thisdir)
        
        [np,nu]=size(Bst); ndof=nu/2;
        dof=(1:ndof)';
        if includebound == 0 % Remove boundary nodes if required
            inner=setdiff(dof,bound);
            A=Ast([inner,inner+ndof],[inner,inner+ndof]);
            B=Bst(:,[inner,inner+ndof]);
        else
            A=Ast;
            B=Bst;
        end
        
        if qmethod > 1
            C = zeros(np);
        end
        
        A = 0.5*(A + A');
        
        r = rank(full(B'));
        [~,~,V] = svd(full(B'));
        Z = V(:,1:r);
        
        QQ=0.5*(Q+Q'); 
        switch diag_type
            case 'diag'
                QQd = diag(diag(Q));
            case 'lump'
                QQd = diag(sum(Q,2));
            otherwise
                QQd = diag(diag(Q));
        end
        
        fprintf('Computing delta_min...')
        del_full = eig(Z'*full(B*(A\(B'))+beta*C)*Z,Z'*full(QQ)*Z);
        del_full = sort(del_full,'ascend');
        del_full(abs(del_full) < 1e-12) = [];
        delmatfull(k,j) = del_full(1);
        del_ful1 = eig(full(B*(A\(B')) + beta*C),full(QQ));
        del_ful1 = sort(del_ful1,'ascend');
        delmatfull1(k,j) = del_ful1(1);
        
        del_diag = eig(Z'*full(B*(A\(B'))+beta*C)*Z,Z'*QQd*Z);
        del_diag = sort(del_diag,'ascend');
        del_diag(abs(del_diag) < 1e-12) = [];
        delmatdiag(k,j) = del_diag(1);
        del_dia1 = eig(full(B*(A\(B')) + beta*C),full(QQd));
        del_dia1 = sort(del_dia1,'ascend');
        delmatdiag1(k,j) = del_dia1(1);
        
        fprintf('Done\n');
    end % for j = 1:nelem
end % for k = 1:nprob

save delmatstuff delmatfull delmatdiag delmatfull1 delmatdiag1

fprintf('Writing table...');
fid = fopen('Deltamat.tex','w+');
fprintf(fid,' & \\qqel{} & \\qpel{} & \\qqqel{}\\\\\n');
fprintf(fid,'\\hline\n');
for k = 1:numprob
    fprintf(fid,'%s & %5.4g & %5.4g & %5.4g\\\\\n',probname{k},delmatfull(k,:));
    %fprintf(fid,'   & %5.4g & %5.4g & %5.4g\\\\\n',delmatdiag(k,:));
end
fprintf('Done\n')
fclose all;