% computes iterations for scaled mass matrices
% Produces Table 4.3 in Pearson, Pestana & Silvester, 2016.
% JJP: 22 Jan, 2016.

% Adds necessary directories to Matlab path
thisdir = pwd;
addpath(genpath(thisdir));
setifisspath;
cd(ifisspath)
setpath;
cd(thisdir)

global amg_grid amg_smoother number_vcycles

prob = 'S3'; % Choose Ifiss problem
grid = 'G5'; % Choose mesh width
elementlist = {'Q1Q1','Q1P0','Q2Q1'};
nelem = length(elementlist);
alphlist = [1:1:10 20:20:100];
numscale = length(alphlist);

% Stores all the iteration counts
itermatdiag = zeros(nelem,numscale);
itermatcheb = zeros(nelem,numscale);
itermatfull = zeros(nelem,numscale);
itermatamgdiag = zeros(nelem,numscale);
itermatamgcheb = zeros(nelem,numscale);
itermatamgfull = zeros(nelem,numscale);
resmatdiag = zeros(nelem,numscale,3);
resmatfull = zeros(nelem,numscale,3);
resmatcheb = zeros(nelem,numscale,3);
resmatamgdiag = zeros(nelem,numscale,3);
resmatamgfull = zeros(nelem,numscale,3);
resmatamgcheb = zeros(nelem,numscale,3);

for elem = 1:nelem
    if strcmp(prob,'S3') == 1
        pb = [prob,'_',grid,'_R3_',elementlist{elem}];
    else
        pb = [prob,'_',grid,'_',elementlist{elem}];
    end
    display(pb)
    batchmode_scalestokes(pb); % Need this for obstacle prob
    close all;
    load batchrun
    cd(thisdir)
    tol = 1e-6; minresmaxit = 200; % for minres
    A = (Ast + Ast')*0.5;
    B = Bst;
    rhs = [fst;gst]; rhs_norm = norm(rhs);
    rhs_norm1 = norm(fst);
    rhs_norm2 = norm(gst);
    
    if qmethod < 2
        K = [A,B';B,-beta*C];
        [m,n] = size(B);
    else
        [m,n] = size(B);
        K = [A,B';B,sparse(m,m)];
    end
    K = 0.5*(K+K');
    
    QQ=0.5*(Q+Q');
    QQd=diag(diag(Q));
    
    number_vcycles = 1;
    amg_grid = amg_grids_setup(Ast);
    smoother_params = amg_smoother_params(amg_grid, 'PGS',2);
    amg_smoother = amg_smoother_setup(amg_grid,smoother_params);
    MA='m_amgzz'; aparams = struct('A',Ast);
    save AMGparams number_vcycles amg_grid smoother_params amg_smoother MA aparams
    
    fprintf('Running MINRES\n')
    for i = 1:numscale;
        fprintf('alpha = %g\n',alphlist(i));
        alpha = alphlist(i);
        
        % Full A, diag Q
        achoice = 2; qchoice = 2;
        it_stokes_driver;
        itermatdiag(elem,i) = length(resvec)-1;
        
        rv = rhs - K*xest;
        resmatdiag(elem,i,1) = norm(rv)/rhs_norm;
        resmatdiag(elem,i,2) = norm(rv(1:n))/rhs_norm1;
        resmatdiag(elem,i,3) = norm(rv(n+1:end))/rhs_norm2;
        
        if resmatdiag(elem,i)>10*tol
            warning('resmat = %g at alpha = %g',resmatdiag(elem,i),alphlist(i));
        end
        clear achoice qchoice resvec xest
        
        % Full A, full Q
        achoice = 2; qchoice = 3;
        it_stokes_driver;
        itermatfull(elem,i) = length(resvec)-1;
        
        rv = rhs - K*xest;
        resmatfull(elem,i,1) = norm(rv)/rhs_norm;
        resmatfull(elem,i,2) = norm(rv(1:n))/rhs_norm1;
        resmatfull(elem,i,3) = norm(rv(n+1:end))/rhs_norm2;
        
        if resmatfull(elem,i)>10*tol
            warning('resmat = %g at alpha = %g for full Q',resmatfull(elem,i),alphlist(i));
        end
        clear achoice qchoice resvec xest
        
        % Full A, cheb Q
        achoice = 2; qchoice = 1;
        it_stokes_driver;
        itermatcheb(elem,i) = length(resvec)-1;

        rv = rhs - K*xest;
        resmatcheb(elem,i,1) = norm(rv)/rhs_norm;
        resmatcheb(elem,i,2) = norm(rv(1:n))/rhs_norm1;
        resmatcheb(elem,i,3) = norm(rv(n+1:end))/rhs_norm2;
        
        if resmatcheb(elem,i)>10*tol
            warning('resmat = %g at alpha = %g for full Q',resmatcheb(elem,i),alphlist(i));
        end
        clear achoice qchoice resvec xest
        
        % AMG A, diag Q
        achoice = 1; qchoice = 2;
        it_stokes_driver;
        itermatamgdiag(elem,i) = length(resvec)-1;
        
        rv = rhs - K*xest;
        resmatamgdiag(elem,i,1) = norm(rv)/rhs_norm;
        resmatamgdiag(elem,i,2) = norm(rv(1:n))/rhs_norm1;
        resmatamgdiag(elem,i,3) = norm(rv(n+1:end))/rhs_norm2;
        
        if resmatamgdiag(elem,i)>10*tol
            warning('resmat = %g at alpha = %g',resmatamgdiag(elem,i),alphlist(i));
        end
        clear achoice qchoice resvec xest
        
        % AMG A, full Q
        achoice = 1; qchoice = 3;
        it_stokes_driver;
        itermatamgfull(elem,i) = length(resvec)-1;
        
        rv = rhs - K*xest;
        resmatamgfull(elem,i,1) = norm(rv)/rhs_norm;
        resmatamgfull(elem,i,2) = norm(rv(1:n))/rhs_norm1;
        resmatamgfull(elem,i,3) = norm(rv(n+1:end))/rhs_norm2;
        
        if resmatamgfull(elem,i)>10*tol
            warning('resmat = %g at alpha = %g for full Q',resamgmatfull(elem,i),alphlist(i));
        end
        clear achoice qchoice resvec xest
        
        % AMG A, cheb Q
        achoice = 1; qchoice = 1;
        it_stokes_driver;
        itermatamgcheb(elem,i) = length(resvec)-1;
        
        rv = rhs - K*xest;
        resmatamgcheb(elem,i,1) = norm(rv)/rhs_norm;
        resmatamgcheb(elem,i,2) = norm(rv(1:n))/rhs_norm1;
        resmatamgcheb(elem,i,3) = norm(rv(n+1:end))/rhs_norm2;
        
        if resmatamgcheb(elem,i)>10*tol
            warning('resmat = %g at alpha = %g for full Q',resmatamgcheb(elem,i),alphlist(i));
        end
        clear achoice qchoice resvec xest
        
        save(['Iter',prob,grid,'.mat'],'alphlist','resmatamgcheb','resmatamgdiag','resmatamgfull','resmatcheb','resmatdiag','resmatfull','itermatamgcheb','itermatamgdiag','itermatamgfull','itermatcheb','itermatdiag','itermatfull')
    end
end % for elem = 1:nelem

fitc = fopen('IterTable.tex','w+');
fprintf(fitc,'& \\multicolumn{6}{|c|}{\\qqel{}} & \\multicolumn{6}{|c|}{\\qpel{}} & \\multicolumn{6}{|c}{\\qqqel{}}\\\\\n');
fprintf(fitc,'$\\alpha$ & 1 & 2 & 3 & 4 & 5 & 6 ');
fprintf(fitc,'& 1 & 4 ');
fprintf(fitc,'& 1 & 2 & 3 & 4 & 5 & 6 \\\\\n\\hline\n');

for i = 1:numscale
    fprintf(fitc,'%i ',alphlist(i));
    fprintf(fitc,'& %i & %i & %i ',itermatfull(1,i),itermatdiag(1,i),itermatcheb(1,i));
    fprintf(fitc,'& %i & %i & %i ',itermatamgfull(1,i),itermatamgdiag(1,i),itermatamgcheb(1,i));
    fprintf(fitc,'& %i ',itermatfull(2,i));
    fprintf(fitc,'& %i ',itermatamgfull(2,i));
    fprintf(fitc,'& %i & %i & %i ',itermatfull(3,i),itermatdiag(3,i),itermatcheb(3,i));
    fprintf(fitc,'& %i & %i & %i \\\\\n',itermatamgfull(3,i),itermatamgdiag(3,i),itermatamgcheb(3,i));
end
fclose all;