% Finds maximum \|r_k\|_2/\|r_0\|_2 and corresponding index into 
% alphavec for different methods. Computes data for Table 5.4 
% in Pearson, Pestana & Silvester, 2016.
%
% JJP: 08 Aug, 2016
%
% NOTE: The user MUST have run Iterations.m for the corresponding 
% problem BEFORE running this m-file.

problist = {'S3','S5'};
gridlist = {'G5'};
problabel = {'regularized cavity','obstacle problem'};
gridlabel = {'$2^{-5}$', '$2^{-6}$', '$2^{-7}$'};
elementlist = {'Q1Q1','Q1P0','Q2Q1'};

mrv = zeros(2,3,6,3);
mri = zeros(2,3,6,3);
mrv2 = zeros(2,3,6,3);
mri2 = zeros(2,3,6,3);
mrv3 = zeros(2,3,6,3);
mri3 = zeros(2,3,6,3);

% Get maximum residual and corresponding index into vector of alpha values
for pr = 1:length(problist)
    for gr = 1:length(gridlist)
        load(['Iter',problist{pr},gridlist{gr},'.mat']);
        
        % Full A, full Q
        [m,i] = max(resmatfull(:,:,1),[],2);
        mrv(pr,gr,1,:) = m;
        mri(pr,gr,1,:) = i;
        
        [m,i] = max(resmatfull(:,:,2),[],2);
        mrv2(pr,gr,1,:) = m;
        mri2(pr,gr,1,:) = i;
        
        [m,i] = max(resmatfull(:,:,3),[],2);
        mrv3(pr,gr,1,:) = m;
        mri3(pr,gr,1,:) = i;
        
        % Full A, Cheb Q
        [m,i] = max(resmatcheb(:,:,1),[],2);
        mrv(pr,gr,2,:) = m;
        mri(pr,gr,2,:) = i;
        
        [m,i] = max(resmatcheb(:,:,2),[],2);
        mrv2(pr,gr,2,:) = m;
        mri2(pr,gr,2,:) = i;
        
        [m,i] = max(resmatcheb(:,:,3),[],2);
        mrv3(pr,gr,2,:) = m;
        mri3(pr,gr,2,:) = i;
        
        % Full A, diag Q
        [m,i] = max(resmatdiag(:,:,1),[],2);
        mrv(pr,gr,3,:) = m;
        mri(pr,gr,3,:) = i;
        
        [m,i] = max(resmatdiag(:,:,2),[],2);
        mrv2(pr,gr,3,:) = m;
        mri2(pr,gr,3,:) = i;
        
        [m,i] = max(resmatdiag(:,:,3),[],2);
        mrv3(pr,gr,3,:) = m;
        mri3(pr,gr,3,:) = i;
        
        % AMG A, full Q
        [m,i] = max(resmatamgfull(:,:,1),[],2);
        mrv(pr,gr,4,:) = m;
        mri(pr,gr,4,:) = i;
        
        [m,i] = max(resmatamgfull(:,:,2),[],2);
        mrv2(pr,gr,4,:) = m;
        mri2(pr,gr,4,:) = i;
        
        [m,i] = max(resmatamgfull(:,:,3),[],2);
        mrv3(pr,gr,4,:) = m;
        mri3(pr,gr,4,:) = i;
        
        % AMG A, Cheb Q
        [m,i] = max(resmatamgcheb(:,:,1),[],2);
        mrv(pr,gr,5,:) = m;
        mri(pr,gr,5,:) = i;
        
        [m,i] = max(resmatamgcheb(:,:,2),[],2);
        mrv2(pr,gr,5,:) = m;
        mri2(pr,gr,5,:) = i;
        
        [m,i] = max(resmatamgcheb(:,:,3),[],2);
        mrv3(pr,gr,5,:) = m;
        mri3(pr,gr,5,:) = i;
        
        % AMG A, diag Q
        [m,i] = max(resmatamgdiag(:,:,1),[],2);
        mrv(pr,gr,6,:) = m;
        mri(pr,gr,6,:) = i;
        
        [m,i] = max(resmatamgdiag(:,:,2),[],2);
        mrv2(pr,gr,6,:) = m;
        mri2(pr,gr,6,:) = i;
        
        [m,i] = max(resmatamgdiag(:,:,3),[],2);
        mrv3(pr,gr,6,:) = m;
        mri3(pr,gr,6,:) = i;
    end
end


% Table
for pr = 1:length(problist)
    for gr = 1:length(gridlist)
        fid = fopen(['../MaximumResidual/MaxRes',problist{pr},gridlist{gr},'.tex'],'w+');
        fprintf(fid,'\\begin{table}\n\\centering\n');
        fprintf(fid,'\\begin{tabular}{l| l l l| l l l}\n');
        fprintf(fid,'$A$ & \\multicolumn{3}{c}{Full} & \\multicolumn{3}{c}{ AMG}\\\\\n');
        fprintf(fid,'$Q$ & Full & Cheb & Diag & Full & Cheb & Diag\\\\\n');
        fprintf(fid,'\\hline\\hline\n');
        for el = 1:length(elementlist)
            fprintf(fid,'%s ',elementlist{el});
            for op = 1:6
                fprintf(fid,'& %3.2g (%i) ',mrv(pr,gr,op,el),alphlist(mri(pr,gr,op,el)));
            end
            fprintf(fid,'\\\\\n');
        end
        fprintf(fid,'\\end{tabular}\n');
        fprintf(fid,'\\caption{Maximum relative residual norms and corresponding $\\alpha$ (in parentheses) for the %s with a mesh width of %s.}\n',problabel{pr},gridlabel{gr});
        fprintf(fid,'\\end{table}\n');
        fclose(fid);
    end
end

% Figure of residual on termination versus alpha
for pr = 1:length(problist)
    for gr = 1:length(gridlist)
        load(['Iter',problist{pr},gridlist{gr},'.mat']);
        
        for el = 1:length(elementlist)
            rmat = zeros(6,15);
            rmat(1,:) = squeeze(resmatfull(el,:,1));
            rmat(2,:) = squeeze(resmatcheb(el,:,1));
            rmat(3,:) = squeeze(resmatdiag(el,:,1));
            rmat(4,:) = squeeze(resmatamgfull(el,:,1));
            rmat(5,:) = squeeze(resmatamgcheb(el,:,1));
            rmat(6,:) = squeeze(resmatamgdiag(el,:,1));
            semilogy(alphlist,rmat');
            xlabel('$\alpha$','Interpreter','latex');
            ylabel('$\|r_k\|/\|r_0\|$','Interpreter','latex');
            legend('full A, full Q','full A, Cheb Q','full A, diag Q','AMG A, full Q','AMG A, Cheb Q','AMG A, diag Q');
            set(gca,'fontsize',18)
            saveas(gcf,['../MaximumResidual/Iter',problist{pr},gridlist{gr},elementlist{el},'.pdf']);
        end
    end
end


