% Computes the Q1 connectivity matrix in Section 3.2 of Pearson,
% Pestana & Silvester, 2016. Converts this to Ifiss connectivity matrix.
% Note that the Ifiss connectivity matrix uses 2x2 macroelements.
%
% Assumes a square grid is used and is slow!
%
% JJP 22 Jan, 2016

thisdir = pwd;
addpath(genpath(thisdir));
setifisspath;
cd(ifisspath)
setpath;
cd(thisdir)

batchmode('S3_G4_R3_Q1Q1'); % Choose a problem
load batchrun
cd(thisdir)

nel = size(ee,1);
nx = sqrt(nel);
ny = sqrt(nel);
[m,~] = size(Bst);
LL = zeros(nx*ny*4,(nx+1)*(ny+1));
Ix = eye(nx); Iy = eye(ny); I = eye(4);

% Bottom row
fprintf('Bottom row...');
LL(:,1) = kron(Iy(:,1),kron(Ix(:,1),I(:,1)));
for i = 1:nx-1
    LL(:,i+1) = kron(Iy(:,1),kron(Ix(:,i),I(:,2)) + kron(Ix(:,i+1),I(:,1)));
end
LL(:,nx+1) = kron(Iy(:,1),kron(Ix(:,nx),I(:,2)));
fprintf('Done\n');

% Middle rows
fprintf('Middle rows...');
for j = 1:ny-1
    LL(:,j*(nx+1)+1) = kron(Iy(:,j),kron(Ix(:,1),I(:,4))) + kron(Iy(:,j+1),kron(Ix(:,1),I(:,1)));
    for i = 1:nx-1
        LL(:,j*(nx+1)+i +1) = kron(Iy(:,j),kron(Ix(:,i),I(:,3))+kron(Ix(:,i+1),I(:,4))) +...
            kron(Iy(:,j+1),kron(Ix(:,i),I(:,2))+kron(Ix(:,i+1),I(:,1)));
    end
    LL(:,j*(nx+1)+nx+1) = kron(Iy(:,j),kron(Ix(:,nx),I(:,3))) + kron(Iy(:,j+1),kron(Ix(:,nx),I(:,2)));
end
fprintf('Done\n');

% Top row
fprintf('Top row...');
LL(:,ny*(nx+1)+1) = kron(Iy(:,ny),kron(Ix(:,1),I(:,4)));
for i = 1:nx-1
    LL(:,ny*(nx+1)+1+i) = kron(Iy(:,ny),kron(Ix(:,i),I(:,3))+kron(Ix(:,i+1),I(:,4)));
end
LL(:,(ny+1)*(nx+1)) = kron(Iy(:,ny),kron(Ix(:,nx),I(:,3)));
fprintf('Done\n');

% Convert indices to 2x2 macroelement for comparison with IFISS
fprintf('Converting to Ifiss connectivity matrix...')
ind = 1:nx*ny*4;
ind = reshape(ind,8,nx*ny/2);

ind2 = [];
for j = 1:ny/2;
    ind2 = [ind2,[(j-1)*nx+(1:nx/2); (j-1/2)*nx+(1:nx/2)]];
end
ind2 = reshape(ind2,nx*ny/2,1);

ind3 = ind(:,ind2);

for j = 2:2:nx*ny/2
    ind3(:,j) = [ind3(5:8,j);ind3(1:4,j)];
end
ind4 = reshape(ind3,nx*ny*4,1);

LIfiss = LL(ind4,:);
fprintf('Done\n');

% Get eigenvectors
fprintf('Computing eigenvectors...')
VQel = [-1 1 1 1; 1 -1 1 1; -1 -1 -1 1; 1 1 -1 1]; % Use eigenvectors in ESW
QDev = zeros(m,4);
onev = ones(sqrt(nel),1);
pm1v = onev; pm1v(2:2:end) = -1;
QDev(:,1) = (LL'*LL)\(LL'*kron(-pm1v,kron(pm1v,VQel(:,1))));
QDev(:,2) = (LL'*LL)\(LL'*kron(onev,kron(pm1v,VQel(:,2))));
QDev(:,3) = (LL'*LL)\(LL'*kron(pm1v,kron(onev,VQel(:,3))));
QDev(:,4) = (LL'*LL)\(LL'*kron(onev,kron(onev,VQel(:,4))));
fprintf('Done\n');