% Computes Q1-P0 Ifiss connectivity matrix and element mass matrix.
% Tests Case II eigenvalues and eigenvectors.
%
% For Pearson, Pestana & Silvester, 2016.
%
% Designed for square domains!
%
% JJP 22 Jan, 2015


thisdir = pwd;
addpath(genpath(thisdir));
setifisspath;
cd(ifisspath)
setpath;
cd(thisdir)

batchmode('S3_G4_R3_Q1P0'); % Choose a problem
load batchrun
cd(thisdir)
[m,n] = size(Bst);

% Get Ifiss Q1 connectivity matrix L
load cavity_grid
[ev,ee,ebound,xyp] = q1p0grid(x,y,xy,mv,bound,mbound);

% Get connectivity matrix L
nel = size(ev,1);
L = eye(nel,nel);

% Compute element mass matrices; based on Ifiss code
nngpt = 4;
x=xy(:,1); y=xy(:,2);

if (nngpt==4)        % 2x2 Gauss points
    gpt=1.0e0/sqrt(3.0e0);
    s(1) = -gpt; t(1) = -gpt; wt(1)=1;
    s(2) =  gpt; t(2) = -gpt; wt(2)=1;
    s(3) =  gpt; t(3) =  gpt; wt(3)=1;
    s(4) = -gpt; t(4) =  gpt; wt(4)=1;
elseif (nngpt==1)   % 1x1 Gauss point
    s(1) =    0; t(1) =    0; wt(1)=4;
else
    error('Check Gauss point integration specification')
end
%
% inner loop over elements
for ivtx = 1:4
    xl_v(:,ivtx) = x(ev(:,ivtx));
    yl_v(:,ivtx) = y(ev(:,ivtx));
end

% Get element mass matrix
mpe = zeros(nel,1,1);
%
% loop over Gauss points
for igpt = 1:nngpt
    sigpt=s(igpt);
    tigpt=t(igpt);
    wght=wt(igpt);
    %  evaluate derivatives etc
    [jac,invjac,phi,dphidx,dphidy] = deriv(sigpt,tigpt,xl_v,yl_v);
    mpe(:,1,1) = mpe(:,1,1) + wght*jac(:);
    % end of Gauss point loop
end

% Get element matrix for Q
Q_el = diag(mpe);

% Expand out to full mass matrix eigenvectors
Q4 = Q_el(1:4,1:4); Q4 = 0.5*(Q4 + Q4');
dqel = eig(Q4,diag(diag(Q4)));
DQel = diag(dqel);

VQel = [-1 1 1 1; 1 -1 1 1; -1 -1 -1 1; 1 1 -1 1];
QDev = zeros(m,4);
for i = 1:4
    QDev(:,i) = (L'*L)\(L'*repmat(VQel(:,i),nel/4,1));
end

% Test eigenvalues and eigenvectors: Q and C
DCel = diag([4 2 2 0]);
fprintf('Error in (Q,C) = %g\n',norm(C*QDev - Q*QDev*DCel));