% Computes Q1-Q1 Ifiss connectivity matrix, element mass matrix and 
% element contributions to B. Tests Case II eigenvalues and eigenvectors.
%
% For Pearson, Pestana & Silvester, 2016.
%
% Designed for square domains!
%
% JJP 22 Jan, 2015

thisdir = pwd;
addpath(genpath(thisdir));
setifisspath;
cd(ifisspath)
setpath;
cd(thisdir)

batchmode('S3_G4_R3_Q1Q1'); % Choose a problem
load batchrun
cd(thisdir)

% Get Ifiss Q1 connectivity matrix L
[ev,ee,ebound,xyp] = q1q1grid(x,y,xy,mv,bound,mbound);
close all;

% Ifiss connectivity matrix
nel = size(ev,1);
[m,n] = size(Bst);
L = sparse(nel*4,m);
for k = 1:nel
    for i = 1:4
        L((k-1)*4+i,ev(k,i)) = 1;
    end
end

% Compute element mass matrices and contributions to B; based on Ifiss code
nngpt = 4;
x=xy(:,1); y=xy(:,2);

if (nngpt==4)        % 2x2 Gauss points
   gpt=1.0e0/sqrt(3.0e0);
   s(1) = -gpt; t(1) = -gpt; wt(1)=1;
   s(2) =  gpt; t(2) = -gpt; wt(2)=1;
   s(3) =  gpt; t(3) =  gpt; wt(3)=1; 
   s(4) = -gpt; t(4) =  gpt; wt(4)=1;
elseif (nngpt==1)   % 1x1 Gauss point
   s(1) =    0; t(1) =    0; wt(1)=4;
else
   error('Check Gauss point integration specification')
end

for ivtx = 1:4
   xl_v(:,ivtx) = x(ev(:,ivtx));
   yl_v(:,ivtx) = y(ev(:,ivtx)); 
end

 mpe = zeros(nel,4,4);
 bxe = zeros(nel,4,4);
 bye = zeros(nel,4,4);

for igpt = 1:nngpt
   sigpt=s(igpt);
   tigpt=t(igpt);
   wght=wt(igpt);
%  evaluate derivatives etc
   [jac,invjac,phi,dphidx,dphidy] = deriv(sigpt,tigpt,xl_v,yl_v);
   for j = 1:4
      for i = 1:4
         mpe(:,i,j) = mpe(:,i,j) + wght*phi(:,i) .*phi(:,j) .*jac(:);
         bxe(:,i,j) = bxe(:,i,j) - wght* dphidx(:,j).*phi(:,i);
         bye(:,i,j) = bye(:,i,j) - wght* dphidy(:,j).*phi(:,i);
      end
   end
% end of Gauss point loop
end  

% Get element matrices for Q, B_x and B_y
Q_el = zeros(nel*4);
Bx_el = zeros(nel,4);
By_el = zeros(nel,4);

% Converts to block diagonal matrices
for k = 1:nel
    ind = (k-1)*4+1:k*4;
    Q_el(ind,ind) = squeeze(mpe(k,:,:));
    Bx_el(ind,ind) = squeeze(bxe(k,:,:));
    By_el(ind,ind) = squeeze(bye(k,:,:));
end

% Compute eigenvalues and eigenvectors of element mass matrix
Q4 = Q_el(1:4,1:4); Q4 = 0.5*(Q4 + Q4');
dqel = eig(Q4,diag(diag(Q4)));
DQel = diag(dqel);
VQel = [-1 1 1 1; 1 -1 1 1; -1 -1 -1 1; 1 1 -1 1]; % Use eigenvectors in ESW

% Expand out to full mass matrix eigenvectors
QDev = zeros(m,4);
for i = 1:4
    QDev(:,i) = (L'*L)\(L'*kron(repmat(VQel(:,i),nel/4,1),VQel(:,i)));
end

% Test eigenvalues and eigenvectors: Q and diag of Q
Q_d = diag(diag(Q));
fprintf('Error in (Q,Q_d) = %g\n',norm(Q*QDev - Q_d*QDev*DQel));

% Test eigenvalues and eigenvectors: diag of Q and C
DCel = DQel; DCel(4,4) = 0;
fprintf('Error in (C,Q_d) = %g\n',norm(C*QDev - Q_d*QDev*DCel));

% Test eigenvalues and eigenvectors: full Q and C
QCel = eye(4); QCel(4,4) = 0;
fprintf('Error in (C,Q) = %g\n',norm(C*QDev - Q*QDev*QCel));

% Find eigenvectors of element matrix that are in nullspace of Bx,By
% Bx_el1 = Bx_el(1:4,1:4);
% By_el1 = By_el(1:4,1:4);
% 
% fprintf('Bx_el'*VQel)
% display(Bx_el1'*VQel)
% fprintf('By_el'*VQel)
% display(By_el1'*VQel)