%SOLVE_OBSTACLE_STOKES solve Stokes problem in obstacle domain
%   IFISS scriptfile: DJS;  9 January 2015.
% Copyright (c) 2009 D.J. Silvester, H.C. Elman, A. Ramage 
% Original code written by M. Wu, 2009

clear variables
%% compute Stokes solution
%% load assembled matrices
gohome
cd datafiles
load obstacle_stokes_nobc.mat
%
fprintf('stokes system ...\n') 
%% boundary conditions
[Ast,Bst,fst,gst] = flowbc(A,B,f,g,xy,bound);
nlres0_norm = norm([fst;gst]);
%
nv=length(fst)/2; np=length(gst); 
if qmethod>1,
   beta=0;
   xst=[Ast,Bst';Bst,sparse(np,np)]\[fst;gst];
elseif qmethod==1
		beta=1/4;     % default parameter
   xst=[Ast,Bst';Bst,-beta*C]\[fst;gst];
elseif qmethod==0
   fprintf('computing pressure stabilized solution...\n')
   xst=[Ast,Bst';Bst,-C]\[fst;gst]; beta=1;
end

%%% plot solution
spc=default('uniform/nonuniform streamlines 1/2 (default uniform)',1);
contourn = default('number of contour lines (default 50)',50);
flowplot09(qmethod,xst,By,Bx,A,xy,xyp,x,y,bound,bndxy,bnde,obs,contourn,spc,33)

%%% estimate errors
        if qmethod==1
        [jmpx,jmpy,els] = stressjmps_q1p0(1,xst,xy,ev,ebound);
        [error_x, error_y, fex, fey, ae] = stokespost_q1p0_p(jmpx,jmpy,els,xy,ev);
        [error_x,error_y] = stokespost_q1p0_bc(ae,fex,fey,...
                                               error_x,error_y,xy,ev,ebound);
        error_div = q1div(xy,ev,xst);
        errorest=sqrt(sum(error_x.^2 + error_y.^2 + error_div.^2));
        fprintf('estimated overall error is %10.6e \n',errorest)
        ee_error=sqrt((error_x.^2 + error_y.^2 + error_div.^2));
        eplotl(ee_error,ev,xy,x,y,34,'Estimated error')
        pause(5), figure(33)
        elseif qmethod == 0
        [jmpx,jmpy,els] = stressjmps_q1q1(1,xst,xy,ev,ebound);
        [error_x, error_y, fex, fey, ae] = stokespost_q1q1_p(xst,jmpx,jmpy,els,xy,ev);
        [error_x,error_y] = stokespost_q1q1_bc(ae,fex,fey,...
                                               error_x,error_y,xy,ev,ebound);
        error_div = q1div(xy,ev,xst);
        errorest=sqrt(sum(error_x.^2 + error_y.^2 + error_div.^2));
        fprintf('estimated overall error is %10.6e \n',errorest)
        ee_error=sqrt((error_x.^2 + error_y.^2 + error_div.^2));
        eplotl(ee_error,ev,xy,x,y,34,'Estimated error')
        pause(5), figure(33)
        elseif qmethod>1,%stokespost,
        else
        error_div = q2div(xy,mv,xst);	
        end

